@@ -29,13 +29,14 @@ arithmic_map = {
 }
 
 class AstPrinter(AstVisitor):
-    def __init__(self, indent: int = 2, arg_newline_cutoff: int = 5):
+    def __init__(self, indent: int = 2, arg_newline_cutoff: int = 5, update_ast_line_nos: bool = False):
         self.result = ''
         self.indent = indent
         self.arg_newline_cutoff = arg_newline_cutoff
         self.ci = ''
         self.is_newline = True
         self.last_level = 0
+        self.curr_line = 1 if update_ast_line_nos else None
 
     def post_process(self) -> None:
         self.result = re.sub(r'\s+\n', '\n', self.result)
@@ -55,33 +56,52 @@ class AstPrinter(AstVisitor):
     def newline(self) -> None:
         self.result += '\n'
         self.is_newline = True
+        if self.curr_line is not None:
+            self.curr_line += 1
 
     def visit_BooleanNode(self, node: mparser.BooleanNode) -> None:
         self.append('true' if node.value else 'false', node)
+        node.lineno = self.curr_line or node.lineno
 
     def visit_IdNode(self, node: mparser.IdNode) -> None:
         assert isinstance(node.value, str)
         self.append(node.value, node)
+        node.lineno = self.curr_line or node.lineno
 
     def visit_NumberNode(self, node: mparser.NumberNode) -> None:
         self.append(str(node.value), node)
+        node.lineno = self.curr_line or node.lineno
+
+    def escape(self, val: str) -> str:
+        return val.translate(str.maketrans({'\'': '\\\'',
+                                            '\\': '\\\\'}))
 
     def visit_StringNode(self, node: mparser.StringNode) -> None:
         assert isinstance(node.value, str)
-        self.append("'" + node.value + "'", node)
+        self.append("'" + self.escape(node.value) + "'", node)
+        node.lineno = self.curr_line or node.lineno
+
+    def visit_FormatStringNode(self, node: mparser.FormatStringNode) -> None:
+        assert isinstance(node.value, str)
+        self.append("f'" + node.value + "'", node)
+        node.lineno = self.curr_line or node.lineno
 
     def visit_ContinueNode(self, node: mparser.ContinueNode) -> None:
         self.append('continue', node)
+        node.lineno = self.curr_line or node.lineno
 
     def visit_BreakNode(self, node: mparser.BreakNode) -> None:
         self.append('break', node)
+        node.lineno = self.curr_line or node.lineno
 
     def visit_ArrayNode(self, node: mparser.ArrayNode) -> None:
+        node.lineno = self.curr_line or node.lineno
         self.append('[', node)
         node.args.accept(self)
         self.append(']', node)
 
     def visit_DictNode(self, node: mparser.DictNode) -> None:
+        node.lineno = self.curr_line or node.lineno
         self.append('{', node)
         node.args.accept(self)
         self.append('}', node)
@@ -89,61 +109,72 @@ class AstPrinter(AstVisitor):
     def visit_OrNode(self, node: mparser.OrNode) -> None:
         node.left.accept(self)
         self.append_padded('or', node)
+        node.lineno = self.curr_line or node.lineno
         node.right.accept(self)
 
     def visit_AndNode(self, node: mparser.AndNode) -> None:
         node.left.accept(self)
         self.append_padded('and', node)
+        node.lineno = self.curr_line or node.lineno
         node.right.accept(self)
 
     def visit_ComparisonNode(self, node: mparser.ComparisonNode) -> None:
         node.left.accept(self)
         self.append_padded(node.ctype, node)
+        node.lineno = self.curr_line or node.lineno
         node.right.accept(self)
 
     def visit_ArithmeticNode(self, node: mparser.ArithmeticNode) -> None:
         node.left.accept(self)
         self.append_padded(arithmic_map[node.operation], node)
+        node.lineno = self.curr_line or node.lineno
         node.right.accept(self)
 
     def visit_NotNode(self, node: mparser.NotNode) -> None:
+        node.lineno = self.curr_line or node.lineno
         self.append_padded('not', node)
         node.value.accept(self)
 
     def visit_CodeBlockNode(self, node: mparser.CodeBlockNode) -> None:
+        node.lineno = self.curr_line or node.lineno
         for i in node.lines:
             i.accept(self)
             self.newline()
 
     def visit_IndexNode(self, node: mparser.IndexNode) -> None:
         node.iobject.accept(self)
+        node.lineno = self.curr_line or node.lineno
         self.append('[', node)
         node.index.accept(self)
         self.append(']', node)
 
     def visit_MethodNode(self, node: mparser.MethodNode) -> None:
+        node.lineno = self.curr_line or node.lineno
         node.source_object.accept(self)
         self.append('.' + node.name + '(', node)
         node.args.accept(self)
         self.append(')', node)
 
     def visit_FunctionNode(self, node: mparser.FunctionNode) -> None:
+        node.lineno = self.curr_line or node.lineno
         self.append(node.func_name + '(', node)
         node.args.accept(self)
         self.append(')', node)
 
     def visit_AssignmentNode(self, node: mparser.AssignmentNode) -> None:
+        node.lineno = self.curr_line or node.lineno
         self.append(node.var_name + ' = ', node)
         node.value.accept(self)
 
     def visit_PlusAssignmentNode(self, node: mparser.PlusAssignmentNode) -> None:
+        node.lineno = self.curr_line or node.lineno
         self.append(node.var_name + ' += ', node)
         node.value.accept(self)
 
     def visit_ForeachClauseNode(self, node: mparser.ForeachClauseNode) -> None:
-        varnames = [x for x in node.varnames]
+        node.lineno = self.curr_line or node.lineno
         self.append_padded('foreach', node)
-        self.append_padded(', '.join(varnames), node)
+        self.append_padded(', '.join(node.varnames), node)
         self.append_padded(':', node)
         node.items.accept(self)
         self.newline()
@@ -151,6 +182,7 @@ class AstPrinter(AstVisitor):
         self.append('endforeach', node)
 
     def visit_IfClauseNode(self, node: mparser.IfClauseNode) -> None:
+        node.lineno = self.curr_line or node.lineno
         prefix = ''
         for i in node.ifs:
             self.append_padded(prefix + 'if', node)
@@ -162,15 +194,18 @@ class AstPrinter(AstVisitor):
         self.append('endif', node)
 
     def visit_UMinusNode(self, node: mparser.UMinusNode) -> None:
+        node.lineno = self.curr_line or node.lineno
         self.append_padded('-', node)
         node.value.accept(self)
 
     def visit_IfNode(self, node: mparser.IfNode) -> None:
+        node.lineno = self.curr_line or node.lineno
         node.condition.accept(self)
         self.newline()
         node.block.accept(self)
 
     def visit_TernaryNode(self, node: mparser.TernaryNode) -> None:
+        node.lineno = self.curr_line or node.lineno
         node.condition.accept(self)
         self.append_padded('?', node)
         node.trueblock.accept(self)
@@ -178,6 +213,7 @@ class AstPrinter(AstVisitor):
         node.falseblock.accept(self)
 
     def visit_ArgumentNode(self, node: mparser.ArgumentNode) -> None:
+        node.lineno = self.curr_line or node.lineno
         break_args = (len(node.arguments) + len(node.kwargs)) > self.arg_newline_cutoff
         for i in node.arguments + list(node.kwargs.values()):
             if not isinstance(i, (mparser.ElementaryNode, mparser.IndexNode)):
@@ -256,6 +292,9 @@ class AstJSONPrinter(AstVisitor):
     def visit_StringNode(self, node: mparser.StringNode) -> None:
         self.gen_ElementaryNode(node)
 
+    def visit_FormatStringNode(self, node: mparser.FormatStringNode) -> None:
+        self.gen_ElementaryNode(node)
+
     def visit_ArrayNode(self, node: mparser.ArrayNode) -> None:
         self._accept('args', node.args)
         self.setbase(node)
