@@ -32,7 +32,6 @@ from mesonbuild.mesonlib import (
 )
 from mesonbuild.compilers import (
     detect_c_compiler, detect_d_compiler, compiler_from_language,
-    GnuLikeCompiler
 )
 from mesonbuild.programs import ExternalProgram
 import mesonbuild.dependencies.base
@@ -169,7 +168,7 @@ class WindowsTests(BasePlatformTests):
             self.assertRebuiltTarget('prog_1')
 
     def test_msvc_cpp17(self):
-        testdir = os.path.join(self.unit_test_dir, '45 vscpp17')
+        testdir = os.path.join(self.unit_test_dir, '44 vscpp17')
 
         env = get_fake_env(testdir, self.builddir, self.prefix)
         cc = detect_c_compiler(env, MachineChoice.HOST)
@@ -222,21 +221,21 @@ class WindowsTests(BasePlatformTests):
     def test_link_environment_variable_lld_link(self):
         env = get_fake_env()
         comp = detect_c_compiler(env, MachineChoice.HOST)
-        if isinstance(comp, GnuLikeCompiler):
+        if comp.get_argument_syntax() == 'gcc':
             raise SkipTest('GCC cannot be used with link compatible linkers.')
         self._check_ld('lld-link', 'c', 'lld-link')
 
     def test_link_environment_variable_link(self):
         env = get_fake_env()
         comp = detect_c_compiler(env, MachineChoice.HOST)
-        if isinstance(comp, GnuLikeCompiler):
+        if comp.get_argument_syntax() == 'gcc':
             raise SkipTest('GCC cannot be used with link compatible linkers.')
         self._check_ld('link', 'c', 'link')
 
     def test_link_environment_variable_optlink(self):
         env = get_fake_env()
         comp = detect_c_compiler(env, MachineChoice.HOST)
-        if isinstance(comp, GnuLikeCompiler):
+        if comp.get_argument_syntax() == 'gcc':
             raise SkipTest('GCC cannot be used with link compatible linkers.')
         self._check_ld('optlink', 'c', 'optlink')
 
@@ -357,7 +356,7 @@ class WindowsTests(BasePlatformTests):
             raise SkipTest('C++ modules is only supported with Visual Studio.')
         if version_compare(os.environ['VSCMD_VER'], '<16.10.0'):
             raise SkipTest('C++ modules are only supported with VS 2019 Preview or newer.')
-        self.init(os.path.join(self.unit_test_dir, '86 cpp modules'))
+        self.init(os.path.join(self.unit_test_dir, '85 cpp modules'))
         self.build()
 
     def test_non_utf8_fails(self):
